#!/bin/bash

# Setup firewall for Apache

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout Configure firewall to allow in http and https traffic
shout
shout =================================================================
shout
pause

if rpm --quiet -q firewalld; then
  # openSUSE or something recent enough to be running firewalld
  shout Adding firewall holes for http and https
  firewall-cmd --permanent --add-service http --add-service https
  shout Reloading firewall policy
  firewall-cmd --reload
else
  shout Using YaST to configure the firewall
  NIC="$( ip route show | grep '^default' | perl -pe 's/^.*?\sdev\s+(\S+)\s*.*$/$1/' )"

  shout 'I need to move your default network device to the EXT zone.'
  if [ "x$NIC" != "x" ]; then
    shout 'From your routing table, it appears to be '"$NIC"'.'
  fi
  NIC="$(prompt "Default network device" "eth0" "$NIC")"

  shout "Moving $NIC to the firewall zone 'EXT'"
  yast2 firewall interfaces add interface="$NIC" zone=EXT

  shout "Adding firewall holes for http and https"
  yast2 firewall services add service=service:apache2,service:apache2-ssl zone=EXT
fi

shout
shout Your firewall has been configured to allow in http and https traffic
shout on ports 80/tcp and 443/tcp.
shout

exit 0

