
# This file is included by the top-level script.
# Do not try to run it on its own, it won't help you.

ARG1="$1"
ARG2="$2"
HERE="$(pwd)"
TEMPFILE=/tmp/zendtopatching
ZTCONF=/opt/zendto/config/zendto.conf
ZTPREFS=/opt/zendto/config/preferences.php
ZTINTERNAL=/opt/zendto/config/internaldomains.conf
ZTCSS=/opt/zendto/www/css/local.css
REGDOMAIN=''
ARCH="$(uname -m)"
# This tomfoolery is to handle people doing "sudo install.sh"
# which is just WRONG.
SRCSTORE="$( eval echo "~root" )"
SRCSTORE="$SRCSTORE/zendto-build-store"

# Tell the user what is happening, in bold
shout() {
  printf '\033[1m'"$*"'\033[0m\n'
}

# Wait a bit. Optional arg is number of seconds. Default=4
# If we are running with --defaults, only wait 2 seconds ever.
pause() {
  DOTS='.............................................................'
  GAPS='                                                             '
  SECS="$1"
  if [ "x$SECS" = "x" ]; then
    SECS=4
  fi
  if [ "x$ALLDEFAULTS" = "xy" ]; then
    SECS=2
  fi
  SPACES=0
  while [ $SECS -gt 0 ];
  do
    printf '\033[1mPausing%.'$SECS's %.'$SPACES's\015Pausing%.'$SECS's\033[0m' "$DOTS" "$GAPS" "$DOTS"
    sleep 1
    printf '\015\033[2K'
    SECS=$((SECS-1))
    SPACES=$((SPACES+1))
  done
}

# Can I run this program on the $PATH?
canrun() {
  command -v "$1" >/dev/null 2>&1
}

# Check I am root!
shout ' '
IAM="$( id -nu )"
if [ "x$IAM" = "xroot" ]; then
  shout "Good, I am root. Bow down before me..."
else
  shout "Sorry, this script must be run as root."
  shout "Please become root with 'sudo su -' or"
  shout "'su -' and then run this again."
  exit 1
fi

# Process the optional '--defaults' command-line argument.
if [ "x$ARG1" = "x--defaults" -o "x$ARG2" = "x--defaults" ]; then
  # We were told what to do.
  ALLDEFAULTS=y
  shout ' '
  shout 'Warning: About to install with all defaults.'
  shout 'Press Ctrl-C now if you did not intend this.'
  pause
else
  # Only set it to no if it isn't already defined.
  # That way if we're running a sub-script, we leave it alone.
  if [ "x$ALLDEFAULTS" = "x" ]; then
    ALLDEFAULTS=n
  fi
fi

# Process the optional '--norepo' command-line argument.
USEREPO=''
if [ "x$ARG1" = "x--norepo" -o "x$ARG2" = "x--norepo" ]; then
  # We were told what to do.
  shout
  shout 'I will not attempt to use the yum/apt repo.'
  shout
  pause
  USEREPO=n
fi

# Prompt the user for a response. Pass in
# 1. prompt string
# 2. default default value to be used if the default is blank
# 3. default value (what we guess it should be)
# Call it like the examples below.
prompt() {
  PROMPT="$1"
  DEFDEF="$2"
  DEF="$3"

  # No suggested value? Use the "default default"
  if [ "x$DEF" = "x" ]; then
    DEF="$DEFDEF"
  fi

  # No default default either? Then don't tell the user anything.
  # Pressing return will result in an empty answer, which may be
  # what we want.
  if [ "x$DEF" = "x" ]; then
    DEFOFFERED=''
  else
    DEFOFFERED=" [Default is $DEF]"
  fi

  if [ "$ALLDEFAULTS" = "y" ]; then
    printf "\033[1m$PROMPT$DEFOFFERED:\033[0m $DEF\n" 1>&2
    ANSWER="$DEF"
  else
    printf "\033[1m$PROMPT$DEFOFFERED:\033[0m " 1>&2
    read ANSWER
    if [ "x$ANSWER" = "x" ]; then
      ANSWER="$DEF"
    fi
  fi

  echo "$ANSWER"
}

# Ask a yes/no question. Return true (0) if they said yes, 1 otherwise.
# Pass in the prompt and the default to offer ('y' or 'n').
# Returns true (0) if they said yes. False (1) otherwise.
yesno() {
  YPROMPT="$1"
  YDEFAULT="$2"

  YANSWER=''
  while [ "x$YANSWER" != "xn" -a "x$YANSWER" != "xy" ]
  do
    YANSWER="$(prompt "$YPROMPT (y/n)" "y" "$YDEFAULT" )"
    # Turn anything into a single lower-case letter
    YANSWER="$(echo "$YANSWER" | cut -c 1 | tr '[:upper:]' '[:lower:]' )"
  done
  if [ "x$YANSWER" = "xy" ]; then
    return 0
  fi
  return 1
}

# Run a script if they answer yes to the question.
# Pass in the prompt and the name of the script to run.
# Don't try to do anything clever on the command-line of the script,
# this function is VERY simple.
runIfYes() {
  RPROMPT="$1"
  RSCRIPT="$2"

  if yesno "$RPROMPT" "y"; then
    # Run the script and return true if it succeeded
    $RSCRIPT && return 0
    # We ran it but it failed. Probably want to bail out.
    return 1
  fi
  # They didn't want to run it. Fine, their choice.
  return 0
}

# Is this an rpm/yum-based system?
isyum() {
    if canrun yum; then
      return 0
    else
      return 1
    fi
}

# Is this an apt-based system?
isapt() {
    if canrun apt || canrun apt-get; then
      return 0
    else
      return 1
    fi
}

# Is this a zypper-based system?
iszypper() {
    if canrun zypper; then
      return 0
    else
      return 1
    fi
}

# Is this a BSD pkg-based system?
ispkg() {
    if canrun pkg; then
      return 0
    else
      return 1
    fi
}

# What is the FQDN of this server?
fullhostname() {
    fqdn=$( hostname --fqdn | tr A-Z a-z )
    # Does this fqdn have at least 2 dots in it,
    # neither of which are at the start or the end?
    if echo $fqdn | grep -q '.\...*\..'; then
      # No quotes, to strip leading and trailing whitespace
      echo $fqdn
    else
      echo "$( prompt "What is the full hostname of this Zendto server" "zendto.example.org" "$fqdn" )"
    fi
}

#
# Give the user some hints
#
shout ' '
shout 'Output from the ZendTo installer itself will look like this.'
shout 'Please do read everything printed that looks like this.'
shout 'There will be some important things in it which you will need to know.'
shout ' '
shout 'When you are prompted for a value, always just press Return'
shout 'to accept the default value suggested, unless either'
shout 'a) you are absolutely sure of what you are doing'
shout ' or'
shout 'b) it is blank or clearly rubbish, which usually means'
shout '   something has gone wrong. If that happens, look back'
shout '   through the output and see if you can work it out.'
shout '   No installer is 100%% perfect in all situations.'
shout ' '
pause 6

# Work out what OS and release we are running.
# This has to go here as it needs the prompt() function, and $OS[VER]
# are used in the installSrpmAndDeps function.
OS=unknown
OSRELEASE=/etc/redhat-release
if [ -f $OSRELEASE ]; then
  if grep -q 'Red *Hat' $OSRELEASE; then
    OS=redhat
  elif grep -q 'CentOS' $OSRELEASE; then
    OS=centos
  elif grep -q 'Rocky' $OSRELEASE; then
    OS=rocky
  fi
  if isyum; then
    rpm -q --quiet perl || {
      shout Just going to install Perl...
      pause
      yum -y install perl
    }
    #rpm -q --quiet wget || {
    #  shout Just going to install wget...
    #  yum -y install wget
    #}
  fi
  OSVERFULL="$( perl -pe 's/^[^\d]+([\d.]+).*?$/$1/' < $OSRELEASE )"
  OSVER="$( echo "$OSVERFULL" | cut -d. -f1 )"
fi
if [ "$OS" = "unknown" ]; then
  OSRELEASE=/etc/debian_version
  if [ -f $OSRELEASE ] && canrun lsb_release; then
    # Should give me the string "ubuntu" or "debian"
    OS="$( lsb_release --id | sed -e 's/^.*:\s*//' | tr '[:upper:]' '[:lower:]' )"
    OSVER="$( lsb_release --release | sed -e 's/^.*:\s*\([0-9]*\).*$/\1/' )"
  fi
fi
if [ "$OS" = "unknown" ]; then
  if canrun lsb_release; then
    # Should give me the string "suse" or similar
    OS="$( lsb_release --id | sed -e 's/^.*:\s*//' | tr '[:upper:]' '[:lower:]' )"
    OSVER="$( lsb_release --release | sed -e 's/^.*:\s*\([0-9]*\).*$/\1/' )"
    # But we need to tell between suse (i.e. sles) and opensuse
    if [ "x$OS" = "xsuse" ]; then
      OS='sles'
    fi
  elif [ -f /etc/os-release ]; then
    # I can just source that file, but that's dangerous
    OSID="$( grep '^ID=' /etc/os-release | head -1 | sed -e 's/#.*$//; s/ *$//; s/^ID="*\([^"]*\)"*/\1/;' )"
    if [[ $OSID =~ sles ]]; then
      OS='sles'
    elif [[ $OSID =~ opensuse ]]; then
      OS='opensuse'
    elif [[ $OSID =~ debian ]]; then
      OS='debian'
    fi
    OSVER="$( grep '^VERSION_ID=' /etc/os-release | head -1 | sed -e 's/#.*$//; s/ *$//; s/^VERSION_ID="*\([^"]*\)"*/\1/; s/\..*$//;' )"
  fi
fi
if [ "$OS" = "unknown" ]; then
  if canrun uname; then
    # Should give me the string "freebsd"
    OS="$( uname -s | tr '[:upper:]' '[:lower:]' )"
    # Just get the major version number (11 or 12)
    OSVER="$( uname -r | sed -e 's/[.-].*$//' )"
  fi
fi

if [ "$OS" = "unknown" ]; then
  OS="$(prompt "Am I running redhat, rocky, centos, ubuntu, debian, sles, opensuse or freebsd" "rocky" "$OS")"
  OS="$(echo "$OS" | tr '[:upper:]' '[:lower:]')" # Lower-case it
fi
# Capitalise first letter of OS (only works in bash, not sh).
# Use bash to do it explicitly if we can.
if which bash >/dev/null 2>&1; then
  # This monster stops sh trying (and failing) to parse the redirects
  # in the bash version of the command, by doing everything in an explicit
  # bash shell.
  export OS
  PRETTYOS="$( bash -c "echo \$(tr '[:lower:]' '[:upper:]' <<< \${OS:0:1})\$(tr '[:upper:]' '[:lower:]' <<< \${OS:1})" )"
else
  PRETTYOS="$OS"
fi
OSVER="$(prompt "$PRETTYOS major release number" "8" "$OSVER")"
ARCH="$(prompt "Architecture" "x86_64" "$ARCH")"
shout "I am running $PRETTYOS release $OSVER on $ARCH"
if [ "x$OS" = "xubuntu" -o "x$OS" = "xdebian" ]; then
  export HERESUB="$HERE/Ubuntu-Debian"
elif [ "x$OS" = "xredhat" -o "x$OS" = "xrocky" -o "x$OS" = "xcentos" ]; then
  export HERESUB="$HERE/CentOS-RedHat"
elif [ "x$OS" = "xsles" -o "x$OS" = "xopensuse" ]; then
  export HERESUB="$HERE/SuSE"
elif [ "x$OS" = "xfreebsd" ]; then
  export HERESUB="$HERE/FreeBSD"
else
  shout ' '
  shout 'Sorry, I only know about RedHat Enterprise, Rocky, CentOS, SUSE Enterprise, openSUSE, Ubuntu, Debian and FreeBSD.'
  shout ' '
  shout 'Exiting...'
  exit 1
fi

# Is SELinux installed?
# If so, is it enabled?
# And what policy is it using?
# May need to install policycoreutils on Ubuntu
if sestatus >/dev/null 2>&1; then
  SELINUX="$(sestatus | grep -i 'selinux status:' | awk '{ print $NF }' | tr '[:upper:]' '[:lower:]')"
  #SELINUX="$(prompt "SELinux status" "enabled" "$SELINUX")"
  if [ "x$SELINUX" = "xdisabled" ]; then
    SELINUXPOLICY=none
  else
    if [ "$OS" = "ubuntu" ]; then
      shout "Installing packages needed for SELinux management"
      apt-get -y policycoreutils
    elif isyum; then
      # CentOS or RedHat 6 and 7
      # SELinux on 6 and 7 needs semanage and other tools,
      # which aren't installed by default
      PCUTILS=policycoreutils-python
      if [ "$OS" = "redhat" -o "$OS" = "rocky" -o "$OS" = "centos" ]; then
        if [ "$OSVER" -ge 8 ]; then
          PCUTILS=policycoreutils-python-utils
        fi
      fi
      if ! command -v semanage >/dev/null 2>&1; then
        shout "Installing packages needed for SELinux management"
        yum -y install "$PCUTILS"
      fi
    fi
    SELINUXPOLICY="$(sestatus | egrep -i 'Policy from config file:|Loaded policy name:' | awk '{ print $NF }')"
    SELINUXPOLICY="$(prompt "SELinux policy" "targeted" "$SELINUXPOLICY")"
  fi
else
  SELINUX=disabled
  SELINUXPOLICY=none
fi

# Install an SRPM file and then all the dependencies that it needs
# installed before we can build it.
# Pass in the path of the SRPM file and the path of the .spec file.
installSrpmAndDeps() {
  SRPM="$1"
  SPEC="$2"

  rpm -Uvh "$SRPM" 2> >(grep -iv 'warning:.*does not exist')
  pause
  if [ "$OS$OSVER" = "redhat5" ]; then
    # yum-builddep on RHEL5 cannot open the php53 SRPM,
    # so we have to do it by hand :-(
    REQS=$(grep '^BuildRequires' "$SPEC" | \
           cut -d\  -f2- | \
           perl -pe 's/ *[<=>]+ *[0-9._-]+//g; s/, */\n/g;')
    if [ "x$REQS" = "x" ]; then
      shout "Failed to find list of required packages for building $SRPM."
      REQS="$( prompt "List of packages required to build $SRPM" "" "$REQS" )"
      REQS="$( echo "$REQS" | perl -pe 's/[, ]+/ /g;' )"
    fi
    pause
    shout "Going to fetch these packages as pre-requisites for building $SRPM:"
    shout "$REQS"
    pause
    yum -y install $REQS
    pause
  else
    # And for everyone else except RHEL5...
    #
    # Well, okay, this doesn't actually work on RHEL7.
    # But it does if you manually install all the nasty deps first.
    # And I ain't writing a recursive script to handle deps of deps of deps...
    yum-builddep -y "$SRPM"
  fi
}

# Set a value in files that look like "key = value".
# Pass in the filename, start-of-comment-character, key-name and new value.
# It will replace the last uncommented setting for the key if there is one.
# Otherwise it will replace the last commented-out setting for the key if there is one.
# Otherwise it will add it to the end of the file.
# N.B. This function does NOT take a backup of the file, it overwrites it.
setCfIni() {
  FILE="$1"
  SEP="$2"
  KEY="$3"
  NEWVALUE="$4"
  
  LASTCOMMENTLINE="$( grep -En "^${SEP} *${KEY}($| *=)" "$FILE" | tail -1 | cut -d: -f1 )"
  LASTUNCOMMENTLINE="$( grep -En "^${KEY} *=" "$FILE" | tail -1 | cut -d: -f1 )"

  if [ "x$LASTUNCOMMENTLINE" != "x" ]; then
    LINENUM="$LASTUNCOMMENTLINE"
  elif [ "x$LASTCOMMENTLINE" != "x" ]; then
    LINENUM="$LASTCOMMENTLINE"
  else
    LINENUM=last
  fi

  if [ "$LINENUM" = "last" ]; then
    echo "Appended: $KEY = $NEWVALUE"
    echo "$KEY = $NEWVALUE" >> "$FILE"
  else
    echo "Replaced line $LINENUM: $KEY = $NEWVALUE"
    # Have to use the file extension on -i even though we don't want to.
    # BSD and Linux sed are incompatible if you don't put an extension.
    # The $'\n' is bash syntax to insert a new-line in the command
    # FreeBSD's non-GNU sed requires this new-line to be there.
    sed -i.bak -e "$LINENUM c \\"$'\n'"$KEY = $NEWVALUE" "$FILE"
    rm -f "$FILE".bak
  fi
}

# Set a value in php.ini files. (These use ';' to mark start of a comment)
# Pass in the filename, key-name and new value.
# N.B. This function does NOT take a backup of the file, it overwrites it.
setphpini() {
  FILE="$1"
  KEY="$2"
  NEWVALUE="$3"

  setCfIni "$FILE" ';' "$KEY" "$NEWVALUE"
}

# Set a value in the zendto.conf file. These use '#' to mark start of a comment
# Pass in the key-name and new value.
# N.B. This function does NOT take a backup of the file, it overwrites it.
setzendtoconf() {
  KEY="$1"
  NEWVALUE="$2"

  setCfIni "$ZTCONF" '#' "$KEY" \""$NEWVALUE"\"
}

# Set a value in Postfix main.cf files. (These use '#' to mark start of a comment)
# Pass in the filename, key-name and new value.
# N.B. This function does NOT take a backup of the file, it overwrites it.
setmaincf() {
  FILE="$1"
  KEY="$2"
  NEWVALUE="$3"

  setCfIni "$FILE" '#' "$KEY" "$NEWVALUE"
}

# Set an SELinux boolean and tell the user what we have set to what
# Pass in the boolean name and the value.
setBool() {
  BOOL="$1"
  TRUEFALSE="$2"

  shout "$BOOL" = "$TRUEFALSE"
  setsebool -P "$BOOL" "$TRUEFALSE"
}

# Set a key in preferences.php to the supplied value.
# The value may need quotes around it.
# This has to cope with commented out values,
# comments at the end of the line,
# quoted and unquoted values for the setting,
# and any variations in whitespace.
setPrefphp () {
    KEY="$1"
    QUOTED="$2"
    VALUE="$3"
    # Have to use '$newvalue' as VALUE may contain a "/"
    if [ "x$QUOTED" = "xnotquoted" ]; then
        perl -pi -e "\$newvalue = '$VALUE'; s/^(\s*['\"]$KEY['\"]\s*=>\s*)[^,]*,/\${1}\$newvalue,/;" "$ZTPREFS"
    else
        perl -pi -e "\$newvalue = '$VALUE'; s/^(\s*['\"]$KEY['\"]\s*=>\s*)['\"][^'\"]*['\"],/\${1}'\$newvalue',/;" "$ZTPREFS"
    fi
    if [ "x$KEY" = "xserverRoot" ]; then
      # Phone home
      curl --fail -F hostdomain="$( hostname -d )" -F zendto="$VALUE" http://zend.to/et-install.php >/dev/null 2>&1
    fi
}

# Read the value of a setting in preferences.php.
# Need to know if the value is quoted or not.
# Returns '' if either it wasn't found or it was set to ''.
getPrefphp () {
    KEY="$1"
    QUOTED="$2"
    # Have to use '$newvalue' as VALUE may contain a "/"
    if [ "x$QUOTED" = "xnotquoted" ]; then
      VALUE="$( perl -ne "/^\s*['\"]${KEY}['\"]\s*=>\s*([^\s,]*)\s*,.*\$/ && print \$1;" "$ZTPREFS" )"
    else
      VALUE="$( perl -ne "/^\s*['\"]${KEY}['\"]\s*=>\s*['\"]([^'\"]*)['\"],.*\$/ && print \$1;" "$ZTPREFS" )"
    fi
    echo "$VALUE"
}

# Do all the user interaction required to configure PHPMailer in
# preferences.php.
# This is here as it's identical for CentOS/RHEL & Ubuntu.
# The settings required are:
# 'SMTPserver'   => '', // Blank, so will use PHP mail(). See above.
# 'SMTPport'     => 25,
# 'SMTPsecure'   => 'tls',
# 'SMTPusername' => '',
# 'SMTPpassword' => '',
configurePHPMailer() {
  # First, check if they have already configured it.
  SMTPserver="$( getPrefphp SMTPserver quoted )"
  if [ "x$SMTPserver" != "x" ]; then
    shout
    shout "It looks like you have already setup your SMTP details,"
    shout "so I will leave them alone."
    shout
    return
  fi

  shout
  shout "Now we need to tell ZendTo how to send email."
  shout "This is done by giving it the details of your"
  shout "organisation's SMTP server."
  shout
  SMTPserver="smtp.$(hostname -d)"
  SMTPserver="$(prompt "Fully-qualified name of your SMTP server" "smtp.your-domain.com" "$SMTPserver")"
  setPrefphp SMTPserver quoted "$SMTPserver"

  # Try to contact SMTP server on 3 most likely ports: 25, 587, 465
  PORTS=''
  for P in 25 587 465; do
    # Some nc/ncat commands don't have the nice -z
    if nc -w 1 "$SMTPserver" $P </dev/null >/dev/null 2>&1; then
      PORTS="${PORTS}${P} "
    fi
  done
  if [ "x$PORTS" != "x" ]; then
    shout Your SMTP server is listening on these ports: $PORTS
    SMTPport="$( echo $PORTS | cut -d\  -f1 )"
  else
    shout 'Your SMTP server does not respond to any of the standard ports'
    shout '(25, 587, 465). You probably got the hostname wrong, which you'
    shout 'will need to fix in preferences.php later. For now, assume it'
    shout 'will be using port 25 unless you know otherwise.'
    SMTPport=''
  fi
  SMTPport="$(prompt "SMTP port number (almost always 25, 587 or 465)" "25" "$SMTPport")"
  setPrefphp SMTPport notquoted "$SMTPport"

  if yesno "Does your SMTP server provide TLS-based secure SMTP?" "y"; then
    SMTPsecure='tls'
  else
    SMTPsecure=''
  fi
  setPrefphp SMTPsecure quoted "$SMTPsecure"

  AUTHDEFAULT=n
  SMTPusername=''
  SMTPpassword=''
  if [ "x$SMTPport" = "x587" ]; then
    # They are using "submission" port, which usually requires auth
    AUTHDEFAULT=y
  fi
  if yesno "Does you need to authenticate to your SMTP server to send email?" "$AUTHDEFAULT"; then
    SMTPusername="$(prompt "SMTP authentication username" "" "$SMTPusername")"
    # Ask for the password without displaying it if we can
    if canrun systemd-ask-password; then
      printf '\033[1m'
      SMTPpassword="$(systemd-ask-password "SMTP authentication password:" )"
      printf '\033[0m'
    else
      SMTPpassword="$(prompt "SMTP authentication password" "" "$SMTPpassword")"
    fi
  fi
  setPrefphp SMTPusername quoted "$SMTPusername"
  setPrefphp SMTPpassword quoted "$SMTPpassword"

  shout
}

# Set up the user's internaldomains.conf file for them.
# I'm only going to try to put their real registered domain name in it.
# They can add anything else.
configureInternalDomains() {
  shout
  # Only do it if they have not changed the file.
  # The only non-blank, non-comment lines in the original are Southampton.
  if sed -e '/^#/ d; /^ *$/ d' "$ZTINTERNAL" | \
     egrep -qv '^(soton|southampton)\.ac';  then
    shout It appears you have changed your internaldomains.conf file
    shout so I will leave it alone.
    shout You must still check that it contains a list of the top-level
    shout email domain names that your organisation uses.
  else
    shout Fetching lists of top-level domains...
    # This allows for FreeBSD's different hostname syntax
    FQDN="$( hostname --long 2>/dev/null || hostname -f )"
    REGDOMAIN="$(
    (
      # Fetch top 3 levels of TLDs and label each line with its level
      # with progress output to stderr
      echo -n 3 1>&2
      curl -s https://data.iana.org/TLD/tlds-alpha-by-domain.txt | sed -e 's/^/1 /'
      echo -n 2 1>&2
      curl -s http://www.surbl.org/static/two-level-tlds | sed -e 's/^/2 /'
      echo -n 1 1>&2
      curl -s http://www.surbl.org/static/three-level-tlds | sed -e 's/^/3 /'
      echo 1>&2
    ) | perl "$HERE"/lib/domainname.pl "$FQDN" )"
    shout "ZendTo needs to know your organisation's email domain name"
    shout "so it can allow external users to drop-off files to internal"
    shout "users."
    shout "All sub-domains of it will also be treated as internal."
    REGDOMAIN="$(prompt "Top-level internal email domain" "your-domain.com" "$REGDOMAIN")"
    # Have to put -i extension to make BSD and Linux sed both work
    # Remove everything from the file except comments and blanks
    sed -i.bak -e '/^[^# ]/ d' "$ZTINTERNAL"
    rm -f "$ZTINTERNAL".bak
    echo "$REGDOMAIN" >> "$ZTINTERNAL"
    shout "If you need to add more domains, edit"
    shout '    '"$ZTINTERNAL"
  fi
  # Phone home - gather the list of domains first
  DOMS="$( sed -e '/^#/ d; /^ *$/ d' "$ZTINTERNAL" | tr -s '[:space:]' ',' )"
  curl --fail -F hostdomain="$( hostname -d )" -F emaildomain="$DOMS" -F os="${OS}${OSVER}" https://zend.to/et-install.php >/dev/null 2>&1
  pause 10
}

# Setup their zendto.conf file as much as we can.
# Settings we target are:
# ServiceTitle
# ServiceLogo (make them enter a word (default=ServiceTitle) or a URL of an image (max 260x60 transparent)
# CopyrightYear (set automatically)
# For the next 2, give the example as "First National Bank"
# OrganizationShortName (First National)
# OrganizationShortType (the Bank)
# EmailSenderAddress (use ServiceTitle and REGDOMAIN as default)
# EmailSubjectTag (default to '[ServiceTitle] ')
configureZendToConf() {
  shout
  shout "Configuring the appearance of your ZendTo site."
  shout "These settings are saved in ${ZTCONF}."
  pause
  SERVTITLE="$(prompt "The name of your ZendTo service" "ZendTo" "ZendTo")"
  setzendtoconf "ServiceTitle" "$SERVTITLE"

  shout "The logo for your ZendTo service can either be the name of the service"
  shout "or better, a complete <img src='....' /> HTML tag for your company"
  shout "logo. Max recommended logo size is 260px x 60px."
  SERVLOGO="$(prompt "The logo for your ZendTo service" "$SERVTITLE" "ZendTo")"
  # If it isn't an HTML tag and is too long, make them shorten it.
  while [[ ! "$SERVLOGO" =~ ^\<.+\>$ ]] && (( ${#SERVLOGO} > 9 )); do
    shout "Longer than 8 or 9 characters probably will not"
    shout "work well in the web interface. Please try something"
    shout "shorter for now, or give an HTML <a> tag for a logo."
    shout "You can change it later in $ZTCONF,"
    shout "or customize the CSS for the logo text or image in"
    shout "$ZTCSS."
    SERVLOGO="$(prompt "The logo for your ZendTo service" "$SERVLOGO" "ZendTo")"
  done
  setzendtoconf "ServiceLogo" "$SERVLOGO"

  setzendtoconf "CopyrightYear" "$( date '+%Y' )"

  shout
  shout "As an example, I will suggest responses as if your"
  shout "organization is 'First National Bank'. That should"
  shout "help you give appropriate answers for the next 2."
  shout
  SHORTNAME="$(prompt "Organization Short Name" "First National" "First National")"
  shout
  shout "Note the presence of 'the' in the next answer!"
  SHORTTYPE="$(prompt "Organization Short Type" "the Bank" "the Bank")"
  setzendtoconf "OrganizationShortName" "$SHORTNAME"
  setzendtoconf "OrganizationShortType" "$SHORTTYPE"

  if [ "x$REGDOMAIN" = "x" ]; then
    REGDOMAIN="$( grep '^[^#]' /opt/zendto/config/internaldomains.conf | head -1 )"
  fi
  EMSEND="support@$REGDOMAIN"
  shout "What address should ZendTo send email from?"
  shout "User's replies will not go there, but email delivery errors will."
  shout "I would recommend your organization's usual IT support contact address."
  EMSEND="$(prompt "Send from" "$EMSEND" "$EMSEND")"
  setzendtoconf "EmailSenderAddress" "$SERVTITLE <${EMSEND}>"

  setzendtoconf "EmailSubjectTag" "[${SERVTITLE}] "
  shout
}


# Mark that we have sourced this file.
ZTFUNCTIONS=1

#
# These are all the things we need to export to subsequent parts of the install.
#
export HERE HERESUB TEMPFILE ARCH SRCSTORE OS OSVER SELINUX REGDOMAIN
export SELINUXPOLICY ZTFUNCTIONS ALLDEFAULTS USEREPO ZTPREFS ZTINTERNAL
export ZTCONF
export -f shout canrun prompt yesno runIfYes pause installSrpmAndDeps
export -f isyum isapt iszypper setCfIni setphpini setmaincf setBool
export -f setPrefphp getPrefphp configurePHPMailer configureInternalDomains
export -f setzendtoconf configureZendToConf fullhostname
